'use strict';
require('source-map-support').install();
var assert = require('power-assert');
var schemaid_1 = require('../src/schemaid');
describe('schema id parser test', function () {
    function test(schemaId, expectedId, isFetchable, fileId, isJsonPath, jsonPath, typenames) {
        assert.equal(assert._expr(assert._capt(assert._capt(schemaId, 'arguments/0/callee/object').getAbsoluteId(), 'arguments/0'), {
            content: 'assert.equal(schemaId.getAbsoluteId(), expectedId)',
            filepath: 'test/schemaid_test.js',
            line: 8
        }), assert._expr(assert._capt(expectedId, 'arguments/1'), {
            content: 'assert.equal(schemaId.getAbsoluteId(), expectedId)',
            filepath: 'test/schemaid_test.js',
            line: 8
        }));
        assert.equal(assert._expr(assert._capt(assert._capt(schemaId, 'arguments/0/callee/object').isFetchable(), 'arguments/0'), {
            content: 'assert.equal(schemaId.isFetchable(), isFetchable)',
            filepath: 'test/schemaid_test.js',
            line: 9
        }), assert._expr(assert._capt(isFetchable, 'arguments/1'), {
            content: 'assert.equal(schemaId.isFetchable(), isFetchable)',
            filepath: 'test/schemaid_test.js',
            line: 9
        }));
        assert.equal(assert._expr(assert._capt(assert._capt(schemaId, 'arguments/0/callee/object').getFileId(), 'arguments/0'), {
            content: 'assert.equal(schemaId.getFileId(), fileId)',
            filepath: 'test/schemaid_test.js',
            line: 10
        }), assert._expr(assert._capt(fileId, 'arguments/1'), {
            content: 'assert.equal(schemaId.getFileId(), fileId)',
            filepath: 'test/schemaid_test.js',
            line: 10
        }));
        assert.equal(assert._expr(assert._capt(assert._capt(schemaId, 'arguments/0/callee/object').isJsonPointerHash(), 'arguments/0'), {
            content: 'assert.equal(schemaId.isJsonPointerHash(), isJsonPath)',
            filepath: 'test/schemaid_test.js',
            line: 11
        }), assert._expr(assert._capt(isJsonPath, 'arguments/1'), {
            content: 'assert.equal(schemaId.isJsonPointerHash(), isJsonPath)',
            filepath: 'test/schemaid_test.js',
            line: 11
        }));
        assert.deepEqual(assert._expr(assert._capt(assert._capt(schemaId, 'arguments/0/callee/object').getJsonPointerHash(), 'arguments/0'), {
            content: 'assert.deepEqual(schemaId.getJsonPointerHash(), jsonPath)',
            filepath: 'test/schemaid_test.js',
            line: 12
        }), assert._expr(assert._capt(jsonPath, 'arguments/1'), {
            content: 'assert.deepEqual(schemaId.getJsonPointerHash(), jsonPath)',
            filepath: 'test/schemaid_test.js',
            line: 12
        }));
        assert.deepEqual(assert._expr(assert._capt(assert._capt(schemaId, 'arguments/0/callee/object').getTypeNames(), 'arguments/0'), {
            content: 'assert.deepEqual(schemaId.getTypeNames(), typenames)',
            filepath: 'test/schemaid_test.js',
            line: 13
        }), assert._expr(assert._capt(typenames, 'arguments/1'), {
            content: 'assert.deepEqual(schemaId.getTypeNames(), typenames)',
            filepath: 'test/schemaid_test.js',
            line: 13
        }));
    }
    it('root schema id', function () {
        test(new schemaid_1.SchemaId('/sampleId', []), '/sampleId#', false, '/sampleId#', false, [], ['SampleId']);
        test(new schemaid_1.SchemaId('/sample2/path/file', []), '/sample2/path/file#', false, '/sample2/path/file#', false, [], [
            'Sample2',
            'Path',
            'File'
        ]);
        test(new schemaid_1.SchemaId('https://example.com:3000/path/to/schema/file', []), 'https://example.com:3000/path/to/schema/file#', true, 'https://example.com:3000/path/to/schema/file#', false, [], [
            'ExampleCom3000',
            'Path',
            'To',
            'Schema',
            'File'
        ]);
        test(new schemaid_1.SchemaId('#/definitions/positiveInteger', []), '#/definitions/positiveInteger', false, '#', true, [
            'definitions',
            'positiveInteger'
        ], [
            'Definitions',
            'PositiveInteger'
        ]);
    });
    it('JSON Schema usage pattern', function () {
        test(new schemaid_1.SchemaId('http://x.y.z/rootschema.json#', []), 'http://x.y.z/rootschema.json#', true, 'http://x.y.z/rootschema.json#', false, [], [
            'XYZ',
            'RootschemaJson'
        ]);
        test(new schemaid_1.SchemaId('#foo', ['http://x.y.z/rootschema.json#']), 'http://x.y.z/rootschema.json#foo', true, 'http://x.y.z/rootschema.json#', false, [], [
            'XYZ',
            'RootschemaJson',
            'Foo'
        ]);
        test(new schemaid_1.SchemaId('otherschema.json', ['http://x.y.z/rootschema.json#']), 'http://x.y.z/otherschema.json#', true, 'http://x.y.z/otherschema.json#', false, [], [
            'XYZ',
            'OtherschemaJson'
        ]);
        test(new schemaid_1.SchemaId('#bar', [
            'otherschema.json',
            'http://x.y.z/rootschema.json#'
        ]), 'http://x.y.z/otherschema.json#bar', true, 'http://x.y.z/otherschema.json#', false, [], [
            'XYZ',
            'OtherschemaJson',
            'Bar'
        ]);
        test(new schemaid_1.SchemaId('t/inner.json#/json/path', [
            'otherschema.json',
            'http://x.y.z/rootschema.json#'
        ]), 'http://x.y.z/t/inner.json#/json/path', true, 'http://x.y.z/t/inner.json#', true, [
            'json',
            'path'
        ], [
            'XYZ',
            'T',
            'InnerJson',
            'Json',
            'Path'
        ]);
        test(new schemaid_1.SchemaId('some://where.else/completely#', ['http://x.y.z/rootschema.json#']), 'some://where.else/completely#', false, 'some://where.else/completely#', false, [], [
            'WhereElse',
            'Completely'
        ]);
    });
});
//# sourceMappingURL=schemaid_test.js.map
